@@ -799,6 +799,7 @@ dkim_canon_init(DKIM *dkim, _Bool tmp, _Bool keep)
 				status = dkim_tmpfile(dkim, &fd, keep);
 				if (status != DKIM_STAT_OK)
 				{
+					gnutls_hash_deinit(sha->sha_hd, sha->sha_out);
 					DKIM_FREE(dkim, sha);
 					return status;
 				}
@@ -847,6 +848,7 @@ dkim_canon_init(DKIM *dkim, _Bool tmp, _Bool keep)
 				status = dkim_tmpfile(dkim, &fd, keep);
 				if (status != DKIM_STAT_OK)
 				{
+					EVP_MD_CTX_free(sha->ctx);
 					DKIM_FREE(dkim, sha);
 					return status;
 				}
