@@ -42,6 +42,7 @@ typedef unsigned long cmap_elem_type;
 #define MY_MAILPARSE_ERR_NO_DOMAIN     8   /* domain cannot be empty */
 #define MY_MAILPARSE_ERR_SEMICOL       9   /* ';' without group syntax */
 #define MY_MAILPARSE_ERR_EMPTY_ADDR   10   /* no local part, no domain */
+#define MY_MAILPARSE_ERR_GUNBALANCED  11   /* group not terminated */
 
 /* a bitmap for the "specials" character class */
 #define	CMAP_NBITS	 	(sizeof(cmap_elem_type) * CHAR_BIT)
@@ -316,7 +317,7 @@ static int end_token(char **cont, char *in)
 			if (parens > 0 || quoted)
 				continue;
 
-			*cont = ++p;
+			*cont = ++p;  // return after the comma
 			return rtc;
 
 		  case '\0':
@@ -331,6 +332,23 @@ static int end_token(char **cont, char *in)
 	}
 }
 
+/*
+* hack_cont -- return one byte before cont, so as to remember that
+* we are inside a group.
+*/
+static int hack_cont(int err, char *entry, char **cont)
+{
+	if (cont && entry && *cont > entry && **cont && (*cont)[-1] == ',')
+		*(--*cont) = ':';
+
+// Group has the same syntax of a header field name, so we cannot issue
+// a missing group termination.
+//	else if (err == 0 && cont && entry && *cont > entry && **cont == 0)
+//		err = MY_MAILPARSE_ERR_GUNBALANCED;
+
+	return err;
+}
+
 /*
 **  DKIM_MAIL_PARSE_C -- extract the local-part and hostname from a mail
 **                       header field, e.g. "From:"
@@ -368,13 +386,19 @@ my_mail_parse_c(char *entry_line, char **user_out,
 	char *line = entry_line;
 	char *w = line;
 	char *e = line + strlen(line);
-	bool angle_bracket = false, seen_at = false, ignore_input = false,
+	bool angle_bracket = false, seen_at = false, source_route = false,
 		seen_group = false;
 
 	/* non-multithreaded clients don't have to initialize */
 	if (inited == 0)
 		my_mail_parse_init();
 
+	if (line && *line == ':')
+	{
+		seen_group = true;
+		++line;
+	}
+
 	for (;;)
 	{
 		err = my_mail_token(line, e, &type, &tok_s, &tok_e, &ws);
@@ -394,7 +418,7 @@ my_mail_parse_c(char *entry_line, char **user_out,
 				else if (angle_bracket)
 					err = MY_MAILPARSE_ERR_ANGLE_BRA;
 			}
-			return err;
+			return seen_group? hack_cont(err, entry_line, cont): err;
 		}
 
 		if (type == '<')
@@ -420,30 +444,33 @@ my_mail_parse_c(char *entry_line, char **user_out,
 					else
 						err = MY_MAILPARSE_ERR_NO_DOMAIN;
 				}
-				return err;
+				return seen_group? hack_cont(err, entry_line, cont): err;
 			}
 			*user_out = NULL;
 			*domain_out = NULL;
 			end_token(cont, tok_e);
-			return MY_MAILPARSE_ERR_ANGLE_BRA;
+			err = MY_MAILPARSE_ERR_ANGLE_BRA;
+			return seen_group? hack_cont(err, entry_line, cont): err;
 		}
 		else if (type == ')' || type == ']')
 		{
 			*user_out = NULL;
 			*domain_out = NULL;
 			end_token(cont, tok_e);
-			return type == ')'? MY_MAILPARSE_ERR_PUNBALANCED:
+			err = type == ')'? MY_MAILPARSE_ERR_PUNBALANCED:
 				MY_MAILPARSE_ERR_SUNBALANCED;
+			return seen_group? hack_cont(err, entry_line, cont): err;
 		}
 		else if (type == '@')
 		{
-			if (w > entry_line && !ignore_input) // have user
+			if (w > entry_line && !source_route) // have user
 			{
 				if (seen_at)
 				{
 					*domain_out = NULL;
 					end_token(cont, tok_e);
-					return MY_MAILPARSE_ERR_MANY_AT;
+					err = MY_MAILPARSE_ERR_MANY_AT;
+					return seen_group? hack_cont(err, entry_line, cont): err;
 				}
 				*w++ = '\0';
 				*domain_out = w;
@@ -453,28 +480,37 @@ my_mail_parse_c(char *entry_line, char **user_out,
 			/*
 			* @example.com must be a source route.
 			*/
-				ignore_input = true;
+				source_route = true;
 		}
-		else if (type == ':' || type == ';')
+		else if (type == ':')
 		{
 			/* group or source-routing */
 			*user_out = NULL;
 			*domain_out = NULL;
 			w = entry_line;
-			ignore_input = false;
-
-			if (!ignore_input) // group
+			if (source_route)
+				source_route = false;
+			else
+				seen_group = true;
+		}
+		else if (type == ';')
+		{
+			if (seen_group)
 			{
-				if (type == ':')
-					seen_group = true;
-				else if (seen_group)
-					seen_group = false;
-				else // lone ';'
+				seen_group = false;
+				//if (*user_out && *domain_out)
 				{
-					end_token(cont, tok_e);
-					return MY_MAILPARSE_ERR_SEMICOL;
+					// end_token(cont, tok_e);
+					//return 0;
 				}
 			}
+			else // lone ';'
+			{
+				*user_out = NULL;
+				*domain_out = NULL;
+				err = MY_MAILPARSE_ERR_SEMICOL;
+				return /* seen_group? hack_cont(err, entry_line, cont): */ err;
+			}
 		}
 		else if (type == '\\')
 		{
@@ -482,10 +518,11 @@ my_mail_parse_c(char *entry_line, char **user_out,
 			if (ch == 0)
 			{
 				*cont = --tok_e;
-				return MY_MAILPARSE_ERR_ESCAPE_EOS;
+				err = MY_MAILPARSE_ERR_ESCAPE_EOS;
+				return seen_group? hack_cont(err, entry_line, cont): err;
 			}
 
-			if (!ignore_input)
+			if (!source_route)
 			{
 				if (*user_out == NULL)
 					*user_out = w;
@@ -499,20 +536,23 @@ my_mail_parse_c(char *entry_line, char **user_out,
 		}
 		else if (type == ',')
 		{
-			*w = 0;
-			*cont = tok_e;
-			if (*domain_out == NULL || **domain_out == 0)
+			if (*domain_out != NULL || *user_out != 0)
 			{
-				if (*user_out == NULL || **user_out == 0)
-					err = MY_MAILPARSE_ERR_EMPTY_ADDR;
-				else
-					err = MY_MAILPARSE_ERR_NO_DOMAIN;
+				*w = 0;
+				*cont = tok_e;
+				if (*domain_out == NULL || **domain_out == 0)
+				{
+					if (*user_out == NULL || **user_out == 0)
+						err = MY_MAILPARSE_ERR_EMPTY_ADDR;
+					else
+						err = MY_MAILPARSE_ERR_NO_DOMAIN;
+				}
+				return seen_group? hack_cont(err, entry_line, cont): err;
 			}
-			return err;
 		}
 		else if (type == '[')
 		{
-			if (*domain_out == w && !ignore_input)
+			if (*domain_out == w && !source_route)
 			{
 				int i_type, nu;
 				char *i_tok_s, *i_tok_e;
@@ -542,7 +582,8 @@ my_mail_parse_c(char *entry_line, char **user_out,
 					{
 						*domain_out = NULL;
 						end_token(cont, line);
-						return MY_MAILPARSE_ERR_BAD_LITERAL;
+						err = MY_MAILPARSE_ERR_BAD_LITERAL;
+						return seen_group? hack_cont(err, entry_line, cont): err;
 					}
 				}
 				*w++ = ']';
@@ -552,10 +593,11 @@ my_mail_parse_c(char *entry_line, char **user_out,
 				*user_out = NULL;
 				*domain_out = NULL;
 				end_token(cont, line);
-				return MY_MAILPARSE_ERR_BAD_LITERAL;
+				err = MY_MAILPARSE_ERR_BAD_LITERAL;
+				return seen_group? hack_cont(err, entry_line, cont): err;
 			}
 		}
-		else if (!ignore_input)
+		else if (!source_route)
 		{
 			bool local_part = false;
 			if (type == '"') // remove quotes
@@ -764,6 +806,13 @@ static const struct addr_test
 	{"!def!xyz%abc@example.com", 1, {{"!def!xyz%abc", "example.com", MY_MAILPARSE_OK}}},
 	{"_somename@example.com", 1, {{"_somename", "example.com", MY_MAILPARSE_OK}}},
 	{"_somename@example.com <_somename@example.com>", 1, {{"_somename", "example.com", MY_MAILPARSE_OK}}},
+
+	// Steffen group syntax, https://mailarchive.ietf.org/arch/msg/ietf-dkim/OTcPHG9uTq3gpXL8hb_ihkbeKyY
+	{"Undisclosed recipients:;< x @ y >", 1, {{"x", "y", MY_MAILPARSE_OK}}},
+	{"A group: a@b, c@d; , stdalone@example.com", 3, {{"a", "b", MY_MAILPARSE_OK},
+		{"c", "d", MY_MAILPARSE_OK}, {"stdalone", "example.com", MY_MAILPARSE_OK}}},
+	{":a@b,c@d;,stdalone@example.com", 3, {{"a", "b", MY_MAILPARSE_OK},
+		{"c", "d", MY_MAILPARSE_OK}, {"stdalone", "example.com", MY_MAILPARSE_OK}}},
 	{NULL, 0, {{NULL, NULL, 0}}}
 };
 
@@ -794,10 +843,10 @@ static int do_testsuite(int verbose)
 			while (line && *line)
 			{
 				int rtc = my_mail_parse_c(line, &user, &domain, &cont);
-				if (rtc == 0 && cont && *cont) // check cont is right after comma
+				if (rtc == 0 && cont && *cont) // check cont is at comma
 				{
 					int ndx = cont - in - 1;
-					assert(t->src[ndx] == ',');
+					assert(t->src[ndx] == ',' || *cont == ':');
 				}
 				if (r >= t->nres)
 				{
@@ -835,6 +884,9 @@ static int do_testsuite(int verbose)
 		++nr;
 	}
 
+	if (verbose && errs)
+		fprintf(stderr, "%d errors\n", errs);
+
 	return errs > 0;
 }
 
@@ -843,7 +895,7 @@ int main(int argc, char **argv)
 	int err;
 	char *domain, *user, *cont;
 
-	if (argc != 2)
+	if (argc < 2)
 	{
 		return do_testsuite(0);
 	}
@@ -854,21 +906,24 @@ int main(int argc, char **argv)
 		return 0;
 	}
 
-	cont = argv[1];
-	while (cont && *cont)
+	for (int i = 1; i < argc; ++i)
 	{
-		err = my_mail_parse_c(cont, &user, &domain, &cont);
+		cont = argv[i];
+		while (cont && *cont)
+		{
+			err = my_mail_parse_c(cont, &user, &domain, &cont);
 
-		if (err)
-		{
-			printf("error %d\n", err);
-			break;
-		}
-		else
-		{
-			printf("user: '%s'\ndomain: '%s'\n", 
-				user ? my_mail_unescape(user) : "null",
-				domain ? my_mail_unescape(domain) : "null");
+			if (err)
+			{
+				printf("%d: error %d\n", i, err);
+				break;
+			}
+			else
+			{
+				printf("%d: user: '%s'\ndomain: '%s'\n", i,
+					user ? my_mail_unescape(user) : "null",
+					domain ? my_mail_unescape(domain) : "null");
+			}
 		}
 	}
 
