@@ -40,6 +40,7 @@ the resulting work.
 #include <stdarg.h>
 #include <ctype.h>
 #include <stdint.h>
+#include <inttypes.h>
 #include <limits.h>
 #include <syslog.h>
 #include <time.h>
@@ -760,7 +761,7 @@ stmt_run_n(db_work_area* dwa, stmt_id sid, var_flag_t bitflag, int count, ...)
 		variable_id const id = part->id;
 		if (id && arglen[id]) // not_used_variable or empty strings don't play
 		{
-			size_t l = ep - p;
+			long unsigned int l = ep - p;
 			int err = odbx_escape(handle, dwa->var[id], arglen[id], p, &l);
 			char *next = p + l;
 			if (err || next >= ep)
@@ -865,7 +866,7 @@ stmt_run_n(db_work_area* dwa, stmt_id sid, var_flag_t bitflag, int count, ...)
 			{
 				uint64_t rows = odbx_rows_affected(result);
 					(*do_report)(LOG_DEBUG,
-						"part #%d of the query affected %ld rows",
+						"part #%d of the query affected %" PRIu64 " rows",
 							r_set, rows);
 			}
 			odbx_result_finish(result);
