@@ -1067,6 +1067,8 @@ int main(int argc, char *argv[])
 	if (fl.verbose)
 		printf("%d from \"%s\"\n", rtc, atdom);
 
+	while (fl.cfc)
+		free(cfc_shift(&fl.cfc));
 	fl_clear_msg_info(&info);
 	return rtc;
 }
