@@ -494,9 +494,9 @@ static int do_read_all_values(void *parm_target[PARM_TARGET_SIZE],
 	{
 		int saveerrno = errno;
 		char *sep = "", *pcwd = "";
+		char cwd[PATH_MAX];
 		if (*fname != '/')
 		{
-			char cwd[PATH_MAX];
 			pcwd = getcwd(cwd, sizeof cwd);
 			if (pcwd == NULL)
 				pcwd = "";
@@ -561,9 +561,14 @@ static int do_read_all_values(void *parm_target[PARM_TARGET_SIZE],
 					(*do_report)(LOG_INFO,
 						"%s was included by %s", name, fname);
 				else if (rtc == -3)
+				{
 					(*do_report)(LOG_ALERT,
 						"Too much recursion at line %d in %s",
 						line_no, fname);
+					rtc = -1;
+				}
+				fclose(fp);
+				vb_clean(&vb);
 				return rtc;
 			}
 			errs += rtc;
@@ -817,6 +822,7 @@ int main(int argc, char const *argv[])
 			int rtc = read_all_values(parm_target, argv[i]);
 			printf("read_all_values(parm, %s) = %d\n", argv[i], rtc);
 			print_parm(parm_target);
+			clear_parm(parm_target);
 			free(parm);
 			first_name = i + 1;
 		}
