@@ -1165,7 +1165,7 @@ int decode_quoted_printable(char *in, unsigned len, transform_stack *vc)
 	if (len == 0)
 		return 0;
 
-	char out[len];
+	char out[len + 1];
 	unsigned outlen = 0;
 	int ch;
 
