@@ -45,9 +45,9 @@ union base_ip_fields
 };
 
 #define BASE_STRUCT_FIELDS                  \
-	char ip;    /* 4 or 6               */ \
-	char ipv4_mapped; /* was IPv6       */ \
-	char nu[6];                            \
+	int8_t ip;    /* 4 or 6               */ \
+	int8_t ipv4_mapped; /* was IPv6       */ \
+	int8_t nu[6];                            \
 	union  base_ip_fields u;
 
 typedef struct ip_u
@@ -86,9 +86,9 @@ typedef struct dmarc_rec
 	unsigned opt_pct: 2;
 	unsigned opt_t: 2;
 	unsigned opt_psd: 2;
-	char found; // 1 if valid record found
-	char adkim, aspf, p, sp, np, pct, t, psd;
-	char nu[7];
+	int8_t found; // 1 if valid record found
+	int8_t adkim, aspf, p, sp, np, pct, t, psd;
+	int8_t nu[7];
 } dmarc_rec;
 
 typedef struct domain_data
@@ -110,10 +110,10 @@ typedef struct tree_walk
 	unsigned ndomains;
 	unsigned nrecords;
 	unsigned policy, org;
-	char effective_p;
-	char top_domain_exists; // -1 not tested, 0 not exist, 1 exists
-	unsigned char attempt;  // 0 on first attempt
-	char nu[5];
+	int8_t effective_p;
+	int8_t top_domain_exists; // -1 not tested, 0 not exist, 1 exists
+	uint8_t attempt;  // 0 on first attempt
+	int8_t nu[5];
 	domain_data dd[];
 } tree_walk;
 
